# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.source import Source
from ...types.worker import source_create_params, source_retrieve_list_params, source_retrieve_add_all_params
from ...types.context.success_response_message import SuccessResponseMessage
from ...types.worker.source_retrieve_list_response import SourceRetrieveListResponse

__all__ = ["SourceResource", "AsyncSourceResource"]


class SourceResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SourceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return SourceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SourceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return SourceResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        config: source_create_params.Config,
        name: str,
        description: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Source:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._put(
            "/worker/source",
            body=maybe_transform(
                {
                    "config": config,
                    "name": name,
                    "description": description,
                },
                source_create_params.SourceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Source,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Source:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._get(
            f"/worker/source/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Source,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SuccessResponseMessage:
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._get(
            "/worker/source",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SuccessResponseMessage,
        )

    def retrieve_add_all(
        self,
        id: str,
        *,
        path: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SuccessResponseMessage:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._get(
            f"/worker/source/{id}/add_all",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"path": path}, source_retrieve_add_all_params.SourceRetrieveAddAllParams),
            ),
            cast_to=SuccessResponseMessage,
        )

    def retrieve_list(
        self,
        id: str,
        *,
        path: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SourceRetrieveListResponse:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._get(
            f"/worker/source/{id}/list",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"path": path}, source_retrieve_list_params.SourceRetrieveListParams),
            ),
            cast_to=SourceRetrieveListResponse,
        )


class AsyncSourceResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSourceResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return AsyncSourceResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSourceResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return AsyncSourceResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        config: source_create_params.Config,
        name: str,
        description: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Source:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._put(
            "/worker/source",
            body=await async_maybe_transform(
                {
                    "config": config,
                    "name": name,
                    "description": description,
                },
                source_create_params.SourceCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Source,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Source:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._get(
            f"/worker/source/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Source,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SuccessResponseMessage:
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._get(
            "/worker/source",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SuccessResponseMessage,
        )

    async def retrieve_add_all(
        self,
        id: str,
        *,
        path: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SuccessResponseMessage:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._get(
            f"/worker/source/{id}/add_all",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"path": path}, source_retrieve_add_all_params.SourceRetrieveAddAllParams
                ),
            ),
            cast_to=SuccessResponseMessage,
        )

    async def retrieve_list(
        self,
        id: str,
        *,
        path: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SourceRetrieveListResponse:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._get(
            f"/worker/source/{id}/list",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"path": path}, source_retrieve_list_params.SourceRetrieveListParams),
            ),
            cast_to=SourceRetrieveListResponse,
        )


class SourceResourceWithRawResponse:
    def __init__(self, source: SourceResource) -> None:
        self._source = source

        self.create = to_raw_response_wrapper(
            source.create,
        )
        self.retrieve = to_raw_response_wrapper(
            source.retrieve,
        )
        self.list = to_raw_response_wrapper(
            source.list,
        )
        self.retrieve_add_all = to_raw_response_wrapper(
            source.retrieve_add_all,
        )
        self.retrieve_list = to_raw_response_wrapper(
            source.retrieve_list,
        )


class AsyncSourceResourceWithRawResponse:
    def __init__(self, source: AsyncSourceResource) -> None:
        self._source = source

        self.create = async_to_raw_response_wrapper(
            source.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            source.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            source.list,
        )
        self.retrieve_add_all = async_to_raw_response_wrapper(
            source.retrieve_add_all,
        )
        self.retrieve_list = async_to_raw_response_wrapper(
            source.retrieve_list,
        )


class SourceResourceWithStreamingResponse:
    def __init__(self, source: SourceResource) -> None:
        self._source = source

        self.create = to_streamed_response_wrapper(
            source.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            source.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            source.list,
        )
        self.retrieve_add_all = to_streamed_response_wrapper(
            source.retrieve_add_all,
        )
        self.retrieve_list = to_streamed_response_wrapper(
            source.retrieve_list,
        )


class AsyncSourceResourceWithStreamingResponse:
    def __init__(self, source: AsyncSourceResource) -> None:
        self._source = source

        self.create = async_to_streamed_response_wrapper(
            source.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            source.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            source.list,
        )
        self.retrieve_add_all = async_to_streamed_response_wrapper(
            source.retrieve_add_all,
        )
        self.retrieve_list = async_to_streamed_response_wrapper(
            source.retrieve_list,
        )



"""Manual chaining (i.e. outside of lattices) of Quarks for RAG ingest"""

import os
import pprint

import dotenv
from halo import Halo

import quarkupy.lib as q
import quarkupy.lib.agent as qa

# Configs
dotenv.load_dotenv()

API_KEY = os.environ.get("QUARK_API_KEY")
OPENAI_API_KEY = os.environ.get("OPENAI_API_KEY")
API_END_POINT = "https://demo.quarklabs.ai/api/quark"

if API_KEY is None or API_KEY == "":
    raise ValueError("QUARK_API_KEY environment variable not set")
if OPENAI_API_KEY is None or OPENAI_API_KEY == "":
    raise ValueError("OPENAI_API_KEY environment variable not set")

SPINNER = "dots"



async def main():
    print(q.__banner__)
    with Halo(text="💠 Getting file list...\n", spinner=SPINNER):
        agent = qa.KnowledgeAgent(
            base_url=API_END_POINT,
            openai_api_key=OPENAI_API_KEY,
        )

        try:
            answer = await agent.ask("How do I get support?")
            pprint.pprint(answer)

        except KeyboardInterrupt:
            print("Interrupted")
        except Exception as e:
            print(f"An error occurred while fetching files: {e}")
            raise e


if __name__ == "__main__":
    import asyncio

    asyncio.run(main())


# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.context import ReferenceDepth, classifier_update_params, classifier_update_partial_params
from ...types.context.classifier import Classifier
from ...types.context.reference_depth import ReferenceDepth
from ...types.context.classifier_list_response import ClassifierListResponse
from ...types.context.success_response_message import SuccessResponseMessage

__all__ = ["ClassifiersResource", "AsyncClassifiersResource"]


class ClassifiersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ClassifiersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return ClassifiersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ClassifiersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return ClassifiersResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Classifier:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._get(
            f"/context/classifiers/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Classifier,
        )

    def update(
        self,
        *,
        model_role_id: str,
        name: str,
        owned_by_identity_id: str,
        reference_depth: ReferenceDepth,
        description: str | NotGiven = NOT_GIVEN,
        parent_classifier_id: str | NotGiven = NOT_GIVEN,
        updated_at: Union[str, datetime] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Classifier:
        """
        Args:
          model_role_id: # Model Role

              Determines which model executes the classifier

          name: # Classifier Name

          owned_by_identity_id: # Owner ID

          reference_depth: # Match Segments or Sentences?

          description: # Classifier Description

              Note: the LLM uses this for matching so be descriptive

          updated_at: # Last Updated

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._put(
            "/context/classifiers",
            body=maybe_transform(
                {
                    "model_role_id": model_role_id,
                    "name": name,
                    "owned_by_identity_id": owned_by_identity_id,
                    "reference_depth": reference_depth,
                    "description": description,
                    "parent_classifier_id": parent_classifier_id,
                    "updated_at": updated_at,
                },
                classifier_update_params.ClassifierUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Classifier,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ClassifierListResponse:
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._get(
            "/context/classifiers",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClassifierListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SuccessResponseMessage:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._delete(
            f"/context/classifiers/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SuccessResponseMessage,
        )

    def retrieve_schema(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._get(
            "/context/classifiers/schema",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def update_partial(
        self,
        id: str,
        *,
        model_role_id: str,
        name: str,
        owned_by_identity_id: str,
        reference_depth: ReferenceDepth,
        description: str | NotGiven = NOT_GIVEN,
        parent_classifier_id: str | NotGiven = NOT_GIVEN,
        updated_at: Union[str, datetime] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Classifier:
        """
        Args:
          model_role_id: # Model Role

              Determines which model executes the classifier

          name: # Classifier Name

          owned_by_identity_id: # Owner ID

          reference_depth: # Match Segments or Sentences?

          description: # Classifier Description

              Note: the LLM uses this for matching so be descriptive

          updated_at: # Last Updated

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._patch(
            f"/context/classifiers/{id}",
            body=maybe_transform(
                {
                    "model_role_id": model_role_id,
                    "name": name,
                    "owned_by_identity_id": owned_by_identity_id,
                    "reference_depth": reference_depth,
                    "description": description,
                    "parent_classifier_id": parent_classifier_id,
                    "updated_at": updated_at,
                },
                classifier_update_partial_params.ClassifierUpdatePartialParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Classifier,
        )


class AsyncClassifiersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncClassifiersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return AsyncClassifiersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncClassifiersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return AsyncClassifiersResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Classifier:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._get(
            f"/context/classifiers/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Classifier,
        )

    async def update(
        self,
        *,
        model_role_id: str,
        name: str,
        owned_by_identity_id: str,
        reference_depth: ReferenceDepth,
        description: str | NotGiven = NOT_GIVEN,
        parent_classifier_id: str | NotGiven = NOT_GIVEN,
        updated_at: Union[str, datetime] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Classifier:
        """
        Args:
          model_role_id: # Model Role

              Determines which model executes the classifier

          name: # Classifier Name

          owned_by_identity_id: # Owner ID

          reference_depth: # Match Segments or Sentences?

          description: # Classifier Description

              Note: the LLM uses this for matching so be descriptive

          updated_at: # Last Updated

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._put(
            "/context/classifiers",
            body=await async_maybe_transform(
                {
                    "model_role_id": model_role_id,
                    "name": name,
                    "owned_by_identity_id": owned_by_identity_id,
                    "reference_depth": reference_depth,
                    "description": description,
                    "parent_classifier_id": parent_classifier_id,
                    "updated_at": updated_at,
                },
                classifier_update_params.ClassifierUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Classifier,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> ClassifierListResponse:
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._get(
            "/context/classifiers",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClassifierListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SuccessResponseMessage:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._delete(
            f"/context/classifiers/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SuccessResponseMessage,
        )

    async def retrieve_schema(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._get(
            "/context/classifiers/schema",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def update_partial(
        self,
        id: str,
        *,
        model_role_id: str,
        name: str,
        owned_by_identity_id: str,
        reference_depth: ReferenceDepth,
        description: str | NotGiven = NOT_GIVEN,
        parent_classifier_id: str | NotGiven = NOT_GIVEN,
        updated_at: Union[str, datetime] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Classifier:
        """
        Args:
          model_role_id: # Model Role

              Determines which model executes the classifier

          name: # Classifier Name

          owned_by_identity_id: # Owner ID

          reference_depth: # Match Segments or Sentences?

          description: # Classifier Description

              Note: the LLM uses this for matching so be descriptive

          updated_at: # Last Updated

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._patch(
            f"/context/classifiers/{id}",
            body=await async_maybe_transform(
                {
                    "model_role_id": model_role_id,
                    "name": name,
                    "owned_by_identity_id": owned_by_identity_id,
                    "reference_depth": reference_depth,
                    "description": description,
                    "parent_classifier_id": parent_classifier_id,
                    "updated_at": updated_at,
                },
                classifier_update_partial_params.ClassifierUpdatePartialParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Classifier,
        )


class ClassifiersResourceWithRawResponse:
    def __init__(self, classifiers: ClassifiersResource) -> None:
        self._classifiers = classifiers

        self.retrieve = to_raw_response_wrapper(
            classifiers.retrieve,
        )
        self.update = to_raw_response_wrapper(
            classifiers.update,
        )
        self.list = to_raw_response_wrapper(
            classifiers.list,
        )
        self.delete = to_raw_response_wrapper(
            classifiers.delete,
        )
        self.retrieve_schema = to_raw_response_wrapper(
            classifiers.retrieve_schema,
        )
        self.update_partial = to_raw_response_wrapper(
            classifiers.update_partial,
        )


class AsyncClassifiersResourceWithRawResponse:
    def __init__(self, classifiers: AsyncClassifiersResource) -> None:
        self._classifiers = classifiers

        self.retrieve = async_to_raw_response_wrapper(
            classifiers.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            classifiers.update,
        )
        self.list = async_to_raw_response_wrapper(
            classifiers.list,
        )
        self.delete = async_to_raw_response_wrapper(
            classifiers.delete,
        )
        self.retrieve_schema = async_to_raw_response_wrapper(
            classifiers.retrieve_schema,
        )
        self.update_partial = async_to_raw_response_wrapper(
            classifiers.update_partial,
        )


class ClassifiersResourceWithStreamingResponse:
    def __init__(self, classifiers: ClassifiersResource) -> None:
        self._classifiers = classifiers

        self.retrieve = to_streamed_response_wrapper(
            classifiers.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            classifiers.update,
        )
        self.list = to_streamed_response_wrapper(
            classifiers.list,
        )
        self.delete = to_streamed_response_wrapper(
            classifiers.delete,
        )
        self.retrieve_schema = to_streamed_response_wrapper(
            classifiers.retrieve_schema,
        )
        self.update_partial = to_streamed_response_wrapper(
            classifiers.update_partial,
        )


class AsyncClassifiersResourceWithStreamingResponse:
    def __init__(self, classifiers: AsyncClassifiersResource) -> None:
        self._classifiers = classifiers

        self.retrieve = async_to_streamed_response_wrapper(
            classifiers.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            classifiers.update,
        )
        self.list = async_to_streamed_response_wrapper(
            classifiers.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            classifiers.delete,
        )
        self.retrieve_schema = async_to_streamed_response_wrapper(
            classifiers.retrieve_schema,
        )
        self.update_partial = async_to_streamed_response_wrapper(
            classifiers.update_partial,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Required, TypedDict

__all__ = ["HandlebarsBaseRunParams"]


class HandlebarsBaseRunParams(TypedDict, total=False):
    input_columns: Required[List[str]]

    ipc_dataset_id: Required[str]

    lattice_id: Required[str]

    template: Required[str]

    opt_rendered_col: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .other import (
    OtherResource,
    AsyncOtherResource,
    OtherResourceWithRawResponse,
    AsyncOtherResourceWithRawResponse,
    OtherResourceWithStreamingResponse,
    AsyncOtherResourceWithStreamingResponse,
)
from .context_insert_objects import (
    ContextInsertObjectsResource,
    AsyncContextInsertObjectsResource,
    ContextInsertObjectsResourceWithRawResponse,
    AsyncContextInsertObjectsResourceWithRawResponse,
    ContextInsertObjectsResourceWithStreamingResponse,
    AsyncContextInsertObjectsResourceWithStreamingResponse,
)
from .context_insert_segments import (
    ContextInsertSegmentsResource,
    AsyncContextInsertSegmentsResource,
    ContextInsertSegmentsResourceWithRawResponse,
    AsyncContextInsertSegmentsResourceWithRawResponse,
    ContextInsertSegmentsResourceWithStreamingResponse,
    AsyncContextInsertSegmentsResourceWithStreamingResponse,
)
from .context_insert_extracted_segments import (
    ContextInsertExtractedSegmentsResource,
    AsyncContextInsertExtractedSegmentsResource,
    ContextInsertExtractedSegmentsResourceWithRawResponse,
    AsyncContextInsertExtractedSegmentsResourceWithRawResponse,
    ContextInsertExtractedSegmentsResourceWithStreamingResponse,
    AsyncContextInsertExtractedSegmentsResourceWithStreamingResponse,
)
from .context_insert_classified_segments import (
    ContextInsertClassifiedSegmentsResource,
    AsyncContextInsertClassifiedSegmentsResource,
    ContextInsertClassifiedSegmentsResourceWithRawResponse,
    AsyncContextInsertClassifiedSegmentsResourceWithRawResponse,
    ContextInsertClassifiedSegmentsResourceWithStreamingResponse,
    AsyncContextInsertClassifiedSegmentsResourceWithStreamingResponse,
)

__all__ = [
    "ContextInsertObjectsResource",
    "AsyncContextInsertObjectsResource",
    "ContextInsertObjectsResourceWithRawResponse",
    "AsyncContextInsertObjectsResourceWithRawResponse",
    "ContextInsertObjectsResourceWithStreamingResponse",
    "AsyncContextInsertObjectsResourceWithStreamingResponse",
    "ContextInsertSegmentsResource",
    "AsyncContextInsertSegmentsResource",
    "ContextInsertSegmentsResourceWithRawResponse",
    "AsyncContextInsertSegmentsResourceWithRawResponse",
    "ContextInsertSegmentsResourceWithStreamingResponse",
    "AsyncContextInsertSegmentsResourceWithStreamingResponse",
    "ContextInsertClassifiedSegmentsResource",
    "AsyncContextInsertClassifiedSegmentsResource",
    "ContextInsertClassifiedSegmentsResourceWithRawResponse",
    "AsyncContextInsertClassifiedSegmentsResourceWithRawResponse",
    "ContextInsertClassifiedSegmentsResourceWithStreamingResponse",
    "AsyncContextInsertClassifiedSegmentsResourceWithStreamingResponse",
    "ContextInsertExtractedSegmentsResource",
    "AsyncContextInsertExtractedSegmentsResource",
    "ContextInsertExtractedSegmentsResourceWithRawResponse",
    "AsyncContextInsertExtractedSegmentsResourceWithRawResponse",
    "ContextInsertExtractedSegmentsResourceWithStreamingResponse",
    "AsyncContextInsertExtractedSegmentsResourceWithStreamingResponse",
    "OtherResource",
    "AsyncOtherResource",
    "OtherResourceWithRawResponse",
    "AsyncOtherResourceWithRawResponse",
    "OtherResourceWithStreamingResponse",
    "AsyncOtherResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .extractor import Extractor as Extractor
from .classifier import Classifier as Classifier
from .reference_depth import ReferenceDepth as ReferenceDepth
from .extractor_list_response import ExtractorListResponse as ExtractorListResponse
from .extractor_update_params import ExtractorUpdateParams as ExtractorUpdateParams
from .classifier_list_response import ClassifierListResponse as ClassifierListResponse
from .classifier_update_params import ClassifierUpdateParams as ClassifierUpdateParams
from .success_response_message import SuccessResponseMessage as SuccessResponseMessage
from .extractor_update_partial_params import ExtractorUpdatePartialParams as ExtractorUpdatePartialParams
from .classifier_update_partial_params import ClassifierUpdatePartialParams as ClassifierUpdatePartialParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .context_insert_object_run_params import ContextInsertObjectRunParams as ContextInsertObjectRunParams
from .context_insert_segment_run_params import ContextInsertSegmentRunParams as ContextInsertSegmentRunParams
from .context_insert_extracted_segment_run_params import (
    ContextInsertExtractedSegmentRunParams as ContextInsertExtractedSegmentRunParams,
)
from .context_insert_classified_segment_run_params import (
    ContextInsertClassifiedSegmentRunParams as ContextInsertClassifiedSegmentRunParams,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo
from .reference_depth import ReferenceDepth

__all__ = ["ExtractorUpdateParams"]


class ExtractorUpdateParams(TypedDict, total=False):
    data_type: Required[Literal["String", "Boolean", "Integer", "Float", "Rating", "Object", "Date", "Label"]]
    """# Type of data to extract"""

    model_role_id: Required[str]
    """# Model Role to use for extraction"""

    name: Required[str]
    """# Extractor Name"""

    owned_by_identity_id: Required[str]
    """# Owner"""

    reference_depth: Required[ReferenceDepth]
    """# Match Segments or Sentences"""

    add_reason: bool
    """# Add reason for extracting this data"""

    add_references: bool
    """# Add references to the extracted data

    Default is true
    """

    config: object

    description: str
    """# Extractor Description

    Note: the LLM uses this to perform the extraction, so be descriptive
    """

    examples: object
    """# Examples of the data to extract"""

    updated_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """# Updated"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .identity_role import IdentityRole as IdentityRole
from .identity_model import IdentityModel as IdentityModel
from .role_create_params import RoleCreateParams as RoleCreateParams
from .role_list_response import RoleListResponse as RoleListResponse
from .role_update_params import RoleUpdateParams as RoleUpdateParams
from .identity_list_response import IdentityListResponse as IdentityListResponse
from .identity_update_params import IdentityUpdateParams as IdentityUpdateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.history import quark_update_params
from ...types.history.quark_history_item import QuarkHistoryItem

__all__ = ["QuarkResource", "AsyncQuarkResource"]


class QuarkResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> QuarkResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return QuarkResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> QuarkResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return QuarkResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> QuarkHistoryItem:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._get(
            f"/history/quark/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QuarkHistoryItem,
        )

    def update(
        self,
        *,
        created_at: Union[str, datetime],
        flow_history_id: str,
        identity_id: str,
        input: object,
        output: object,
        quark_history_id: str,
        registry_qrn: str,
        state: object,
        status: Literal["New", "Scheduled", "Running", "OutputStaged", "Completed", "Failed"],
        registry_id: str | NotGiven = NOT_GIVEN,
        runner_task_id: str | NotGiven = NOT_GIVEN,
        supervisor_task_id: str | NotGiven = NOT_GIVEN,
        worker_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> QuarkHistoryItem:
        """
        Args:
          created_at: The timestamp indicating when the Quark was created.

          flow_history_id: Identifier of the associated Flow.

          identity_id: Identity of Quark runner

          input: Input data associated with the Quark, stored as a JSON value.

          output: Output data produced by the Quark execution, stored as a JSON value.

          quark_history_id: Unique identifier for the Quark.

          registry_qrn: User-facing fully qualified identifier for the registry where the Quark is
              defined.

          state: Quark State

          status: Represents the status/stage of a Quark instance

          registry_id: Registry ID of the database entry

          runner_task_id: Runner [WorkerTask] id Optional, as there are stages when no runner is assigned

          supervisor_task_id: Supervisor [WorkerTask] id Optional, as there are stages when no supervisor is
              assigned

          worker_id: Runner [WorkerTask] id Optional, as there are stages when no worker is assigned

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._put(
            "/history/quark",
            body=maybe_transform(
                {
                    "created_at": created_at,
                    "flow_history_id": flow_history_id,
                    "identity_id": identity_id,
                    "input": input,
                    "output": output,
                    "quark_history_id": quark_history_id,
                    "registry_qrn": registry_qrn,
                    "state": state,
                    "status": status,
                    "registry_id": registry_id,
                    "runner_task_id": runner_task_id,
                    "supervisor_task_id": supervisor_task_id,
                    "worker_id": worker_id,
                },
                quark_update_params.QuarkUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QuarkHistoryItem,
        )


class AsyncQuarkResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncQuarkResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return AsyncQuarkResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncQuarkResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return AsyncQuarkResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> QuarkHistoryItem:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._get(
            f"/history/quark/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QuarkHistoryItem,
        )

    async def update(
        self,
        *,
        created_at: Union[str, datetime],
        flow_history_id: str,
        identity_id: str,
        input: object,
        output: object,
        quark_history_id: str,
        registry_qrn: str,
        state: object,
        status: Literal["New", "Scheduled", "Running", "OutputStaged", "Completed", "Failed"],
        registry_id: str | NotGiven = NOT_GIVEN,
        runner_task_id: str | NotGiven = NOT_GIVEN,
        supervisor_task_id: str | NotGiven = NOT_GIVEN,
        worker_id: str | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> QuarkHistoryItem:
        """
        Args:
          created_at: The timestamp indicating when the Quark was created.

          flow_history_id: Identifier of the associated Flow.

          identity_id: Identity of Quark runner

          input: Input data associated with the Quark, stored as a JSON value.

          output: Output data produced by the Quark execution, stored as a JSON value.

          quark_history_id: Unique identifier for the Quark.

          registry_qrn: User-facing fully qualified identifier for the registry where the Quark is
              defined.

          state: Quark State

          status: Represents the status/stage of a Quark instance

          registry_id: Registry ID of the database entry

          runner_task_id: Runner [WorkerTask] id Optional, as there are stages when no runner is assigned

          supervisor_task_id: Supervisor [WorkerTask] id Optional, as there are stages when no supervisor is
              assigned

          worker_id: Runner [WorkerTask] id Optional, as there are stages when no worker is assigned

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._put(
            "/history/quark",
            body=await async_maybe_transform(
                {
                    "created_at": created_at,
                    "flow_history_id": flow_history_id,
                    "identity_id": identity_id,
                    "input": input,
                    "output": output,
                    "quark_history_id": quark_history_id,
                    "registry_qrn": registry_qrn,
                    "state": state,
                    "status": status,
                    "registry_id": registry_id,
                    "runner_task_id": runner_task_id,
                    "supervisor_task_id": supervisor_task_id,
                    "worker_id": worker_id,
                },
                quark_update_params.QuarkUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=QuarkHistoryItem,
        )


class QuarkResourceWithRawResponse:
    def __init__(self, quark: QuarkResource) -> None:
        self._quark = quark

        self.retrieve = to_raw_response_wrapper(
            quark.retrieve,
        )
        self.update = to_raw_response_wrapper(
            quark.update,
        )


class AsyncQuarkResourceWithRawResponse:
    def __init__(self, quark: AsyncQuarkResource) -> None:
        self._quark = quark

        self.retrieve = async_to_raw_response_wrapper(
            quark.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            quark.update,
        )


class QuarkResourceWithStreamingResponse:
    def __init__(self, quark: QuarkResource) -> None:
        self._quark = quark

        self.retrieve = to_streamed_response_wrapper(
            quark.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            quark.update,
        )


class AsyncQuarkResourceWithStreamingResponse:
    def __init__(self, quark: AsyncQuarkResource) -> None:
        self._quark = quark

        self.retrieve = async_to_streamed_response_wrapper(
            quark.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            quark.update,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .ml_model_role import MlModelRole as MlModelRole
from .role_list_response import RoleListResponse as RoleListResponse
from .role_update_params import RoleUpdateParams as RoleUpdateParams
from .role_update_partial_params import RoleUpdatePartialParams as RoleUpdatePartialParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from .quark_tag import QuarkTag
from ...._models import BaseModel
from .schema_info import SchemaInfo
from .described_input_field import DescribedInputField

__all__ = ["LatticeRegistryItem", "Edge", "Node"]


class Edge(BaseModel):
    source_node_id: str

    target_node_id: str


class Node(BaseModel):
    constants: Dict[str, object]

    lattice_to_quark_input_map: Dict[str, str]

    name: str

    node_id: str

    quark_reg_id: str

    description: Optional[str] = None


class LatticeRegistryItem(BaseModel):
    author: str

    created_at: datetime

    edges: List[Edge]

    flow_registry_id: str

    hidden: bool

    identifier: str

    inputs: List[DescribedInputField]

    lattice_type: Literal["Ingest", "Inference", "Other"]

    name: str

    nodes: List[Node]

    tags: List[QuarkTag]

    version: str

    description: Optional[str] = None

    output_schema: Optional[SchemaInfo] = None
    """API-Friendly representation of a [Schema]"""

    updated_at: Optional[datetime] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .role import (
    RoleResource,
    AsyncRoleResource,
    RoleResourceWithRawResponse,
    AsyncRoleResourceWithRawResponse,
    RoleResourceWithStreamingResponse,
    AsyncRoleResourceWithStreamingResponse,
)
from .members import (
    MembersResource,
    AsyncMembersResource,
    MembersResourceWithRawResponse,
    AsyncMembersResourceWithRawResponse,
    MembersResourceWithStreamingResponse,
    AsyncMembersResourceWithStreamingResponse,
)
from .identity import (
    IdentityResource,
    AsyncIdentityResource,
    IdentityResourceWithRawResponse,
    AsyncIdentityResourceWithRawResponse,
    IdentityResourceWithStreamingResponse,
    AsyncIdentityResourceWithStreamingResponse,
)

__all__ = [
    "IdentityResource",
    "AsyncIdentityResource",
    "IdentityResourceWithRawResponse",
    "AsyncIdentityResourceWithRawResponse",
    "IdentityResourceWithStreamingResponse",
    "AsyncIdentityResourceWithStreamingResponse",
    "RoleResource",
    "AsyncRoleResource",
    "RoleResourceWithRawResponse",
    "AsyncRoleResourceWithRawResponse",
    "RoleResourceWithStreamingResponse",
    "AsyncRoleResourceWithStreamingResponse",
    "MembersResource",
    "AsyncMembersResource",
    "MembersResourceWithRawResponse",
    "AsyncMembersResourceWithRawResponse",
    "MembersResourceWithStreamingResponse",
    "AsyncMembersResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .api_keys import (
    APIKeysResource,
    AsyncAPIKeysResource,
    APIKeysResourceWithRawResponse,
    AsyncAPIKeysResourceWithRawResponse,
    APIKeysResourceWithStreamingResponse,
    AsyncAPIKeysResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["ProfileResource", "AsyncProfileResource"]


class ProfileResource(SyncAPIResource):
    @cached_property
    def api_keys(self) -> APIKeysResource:
        return APIKeysResource(self._client)

    @cached_property
    def with_raw_response(self) -> ProfileResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return ProfileResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ProfileResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return ProfileResourceWithStreamingResponse(self)


class AsyncProfileResource(AsyncAPIResource):
    @cached_property
    def api_keys(self) -> AsyncAPIKeysResource:
        return AsyncAPIKeysResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncProfileResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return AsyncProfileResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncProfileResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return AsyncProfileResourceWithStreamingResponse(self)


class ProfileResourceWithRawResponse:
    def __init__(self, profile: ProfileResource) -> None:
        self._profile = profile

    @cached_property
    def api_keys(self) -> APIKeysResourceWithRawResponse:
        return APIKeysResourceWithRawResponse(self._profile.api_keys)


class AsyncProfileResourceWithRawResponse:
    def __init__(self, profile: AsyncProfileResource) -> None:
        self._profile = profile

    @cached_property
    def api_keys(self) -> AsyncAPIKeysResourceWithRawResponse:
        return AsyncAPIKeysResourceWithRawResponse(self._profile.api_keys)


class ProfileResourceWithStreamingResponse:
    def __init__(self, profile: ProfileResource) -> None:
        self._profile = profile

    @cached_property
    def api_keys(self) -> APIKeysResourceWithStreamingResponse:
        return APIKeysResourceWithStreamingResponse(self._profile.api_keys)


class AsyncProfileResourceWithStreamingResponse:
    def __init__(self, profile: AsyncProfileResource) -> None:
        self._profile = profile

    @cached_property
    def api_keys(self) -> AsyncAPIKeysResourceWithStreamingResponse:
        return AsyncAPIKeysResourceWithStreamingResponse(self._profile.api_keys)



"""Manual chaining (i.e. outside of lattices) of Quarks for RAG ingest"""

import os
import pprint
import datetime
from typing import Union
from urllib.parse import urlparse

import dotenv
import uuid_utils as uuid  # Replace standard uuid with uuid_utils for UUID7
from halo import Halo

import quarkupy as q
import quarkupy.lib.runner as qr

# Configs
dotenv.load_dotenv()

API_KEY = os.environ.get("QUARK_API_KEY")
API_END_POINT = "http://local.quarkdev.co:8080/api/quark"
OPENAI_API_KEY = os.environ.get("OPENAI_API_KEY")
AWS_ACCESS_KEY_ID = os.environ.get("AWS_ACCESS_KEY_ID")
AWS_ACCESS_KEY_SECRET = os.environ.get("AWS_ACCESS_KEY_SECRET")
AWS_ENDPOINT = os.environ.get("AWS_ENDPOINT")
S3_SOURCE_URL = os.environ.get("S3_SOURCE_URL")
VECTOR_DB_TABLE = os.environ.get("VECTOR_DB_TABLE")

if API_KEY is None or API_KEY == "":
    raise ValueError("QUARK_API_KEY environment variable not set")
if OPENAI_API_KEY is None or API_KEY == "":
    raise ValueError("OPENAI_API_KEY environment variable not set")
if AWS_ACCESS_KEY_ID is None or API_KEY == "":
    raise ValueError("AWS_ACCESS_KEY_ID environment variable not set")
if AWS_ACCESS_KEY_SECRET is None or API_KEY == "":
    raise ValueError("AWS_ACCESS_KEY_SECRET environment variable not set")
if AWS_ENDPOINT is None or API_KEY == "":
    raise ValueError("AWS_ENDPOINT environment variable not set")
if S3_SOURCE_URL is None or API_KEY == "":
    raise ValueError("S3_SOURCE_URL environment variable not set")
if VECTOR_DB_TABLE is None or API_KEY == "":
    raise ValueError("VECTOR_DB_TABLE environment variable not set")
if not S3_SOURCE_URL.startswith("s3://") or API_KEY == "":
    raise ValueError("S3_SOURCE_URL must start with s3://")

PRINT_DATASETS = False
PRINT_QUARK_METRICS = True
PRINT_QUARK_HISTORY = False
SPINNER = "dots"

host_url = urlparse(API_END_POINT)
url_root = f"{host_url.scheme}://{host_url.netloc}"


# Mock lattice id for grouping Quarks
lattice_id = uuid.uuid7().__str__()
lattice_identifier = "lattice:rag_demo_ingest"


lattice_history = qr.FlowHistoryItem(
    flow_history_id=lattice_id.__str__(),
    registry_qrn=lattice_identifier,
    created_at=datetime.datetime.now(datetime.timezone.utc),
    updated_at=datetime.datetime.now(datetime.timezone.utc),
    nodes=[],
    edges=[],
    input=None,
    output=None,
    metrics=None,
    status="New",
    identity_id="019738eb-eb84-77d1-b0be-1514c0a9ed6a",
)


async def util_save_lattice_history():
    """
    Saves the Quark history (status) on the Quark services via a REST call. This call is an "upsert" call, meaning
    if a history item with the same `quark_id` exists, it will be updated with the new status. If not, a new history
    item will be created.

    :raises Exception: If there is an error during the API operation.
    """
    api = q.AsyncClient(api_key=API_KEY, base_url=API_END_POINT)
    await api.history.flow.update(**lattice_history.model_dump())
    await api.close()


printer = pprint.PrettyPrinter(indent=2)


async def run_quark(q) -> Union[uuid.UUID, None]:
    """Common control flow for running quarks"""
    try:
        await q.start()
        await q.execute()
        print(f"💠 Quark ID: {q.quark_id} - more: {url_root}/q/history/quark/{q.quark_id}")

        # Runs are asynchronous, so we need to poll until the run is complete
        done = await q.poll_until_complete()

        lattice_history.nodes.append(q.history.quark_history_id)

        if PRINT_QUARK_HISTORY:
            printer.pprint(done.model_dump())

        if PRINT_QUARK_METRICS:
            printer.pprint(done.state)

        try:
            print(
                f"💠 Got dataset with ID: {done.output['dataset_uuid']} - preview: {url_root}/q/data-manager/quark-outputs/{done.output['dataset_uuid']}"
            )
            if PRINT_DATASETS:
                t = await q.read_output_dataset()
                if t is not None:
                    printer.pprint(t.to_pandas())
                else:
                    print("No dataset produced")

            return done.output["dataset_uuid"]
        except KeyError:
            return None

    except Exception as e:
        lattice_history.status = "Failed"
        await util_save_lattice_history()

        printer.pprint(e)
        raise e


async def step1_load_raw() -> uuid.UUID:
    with Halo(text="💠 Loading files from S3...\n", spinner=SPINNER):
        q_input = qr.S3ReadWholeFileQuarkInput(
            lattice_id=lattice_id.__str__(),
            url=S3_SOURCE_URL,
            access_key_id=AWS_ACCESS_KEY_ID,
            access_key_secret=AWS_ACCESS_KEY_SECRET,
            opt_endpoint=AWS_ENDPOINT,
        )

        q = qr.S3ReadWholeFileQuark(
            lattice_id=lattice_id, quark_input=q_input, QUARK_API_KEY=API_KEY, BASE_URL=API_END_POINT
        )

        return await run_quark(q)


async def step2_extract_text(dataset_uuid: uuid.UUID) -> uuid.UUID:
    with Halo(text="💠 Extracting text from files...\n", spinner=SPINNER):
        q_input = qr.DocExtractQuarkInput(
            lattice_id=lattice_id.__str__(),
            ipc_dataset_id=dataset_uuid.__str__(),
            opt_generate_page_images=False,
            opt_generate_picture_images=False,
            opt_do_ocr=True,
            opt_do_table_structure=True
        )

        q = qr.DocExtractQuark(
            lattice_id=lattice_id, quark_input=q_input, QUARK_API_KEY=API_KEY, BASE_URL=API_END_POINT
        )

        return await run_quark(q)


async def step3_chunk_text(dataset_uuid: uuid.UUID) -> uuid.UUID:
    with Halo(text="💠 Chunking text...\n", spinner=SPINNER):
        q_input = qr.DocChunkerQuarkInput(
            lattice_id=lattice_id.__str__(), ipc_dataset_id=dataset_uuid.__str__(), opt_drop_cols=["ccs_json"]
        )

        q = qr.DocChunkerQuark(
            lattice_id=lattice_id, quark_input=q_input, QUARK_API_KEY=API_KEY, BASE_URL=API_END_POINT
        )

        return await run_quark(q)


async def step4_make_classifier_prompt(dataset_uuid: uuid.UUID) -> uuid.UUID:
    with Halo(text="💠 Creating classifier and redaction prompt...\n", spinner=SPINNER):
        template = """
        Reply only with JSON undecorated or escaped.
        
        Input text:
        ```
        {{text}}
        ```
        """

        q_input = qr.TextTemplateInput(
            lattice_id=lattice_id.__str__(),
            ipc_dataset_id=dataset_uuid.__str__(),
            template=template,
            input_columns=["text"],
            opt_rendered_col="prompt",
        )

        q = qr.TextTemplateBaseQuark(
            lattice_id=lattice_id, quark_input=q_input, QUARK_API_KEY=API_KEY, BASE_URL=API_END_POINT
        )

        return await run_quark(q)


async def step5_inference(dataset_uuid: uuid.UUID) -> uuid.UUID:
    with Halo(text="💠 Running OpenAI Inference...\n", spinner=SPINNER):
        q_input = qr.OpenAICompletionsInput(
            lattice_id=lattice_id.__str__(),
            ipc_dataset_id=dataset_uuid.__str__(),
            api_key=OPENAI_API_KEY,
            opt_prompt_column="prompt",
            opt_json_output=True,
            opt_system_prompt="""
                            You will be provided with a section of text from various technical documents from a software
                            company's website.
                            Your goal is to extract a list of tags that describe the text element. If more than one matches,
                            select all that apply. Tags overview
                            
                            {
                              "tags": "Subject area addressed in text e.g. 'case_study', 'product_overview', 'sales_materials', 'whitepaper', 'blog_post', 'press_release', 'news_article', 'technical_documentation', 'legal_documentation', 'financial_documentation', 'other'",
                            }
                            
                            Output the tags in json format, with an array of tags that you can extract from the text.
                            """,
        )

        q = qr.OpenAICompletionBaseQuark(
            lattice_id=lattice_id, quark_input=q_input, QUARK_API_KEY=API_KEY, BASE_URL=API_END_POINT
        )

        return await run_quark(q)


async def step6_embeddings(dataset_uuid: uuid.UUID) -> uuid.UUID:
    with Halo(text="💠 Running OpenAI Embeddings...\n", spinner=SPINNER):
        q_input = qr.OpenAIEmbeddingsInput(
            lattice_id=lattice_id.__str__(),
            ipc_dataset_id=dataset_uuid.__str__(),
            api_key=OPENAI_API_KEY,
        )

        q = qr.OpenAIEmbeddingsQuark(
            lattice_id=lattice_id, quark_input=q_input, QUARK_API_KEY=API_KEY, BASE_URL=API_END_POINT
        )

        return await run_quark(q)


async def step7_lancedb_ingest(dataset_uuid: uuid.UUID) -> bool:
    with Halo(text="💠 Running LanceDB Ingest...\n", spinner=SPINNER):
        q_input = qr.VectorDBIngestInput(
            lattice_id=lattice_id.__str__(),
            ipc_dataset_id=dataset_uuid.__str__(),
            table_name=VECTOR_DB_TABLE,
        )

        q = qr.VectorDBIngestQuark(
            lattice_id=lattice_id, quark_input=q_input, QUARK_API_KEY=API_KEY, BASE_URL=API_END_POINT
        )

        await run_quark(q)
        return True


async def main():
    print(q.lib.__banner__)
    print(f"💠 Running lattice ID: {lattice_id} ({API_END_POINT}/history/lattice/{lattice_id})")

    lattice_history.status = "Running"
    await util_save_lattice_history()

    try:
        #await step7_lancedb_ingest(
        await step6_embeddings("019760f5-04ce-7ac0-b11a-6cba17424dab"
            #await step5_inference("019760e1-3132-7a20-8bbc-1398a228eaa3"
                #await step4_make_classifier_prompt(
                #    await step3_chunk_text(
                #       await step2_extract_text(
                #            await step1_load_raw()
                #        )
                #    )
                #)
            #)
        )
        #)

        lattice_history.status = "Completed"
        await util_save_lattice_history()

        print("Done! 🎉")
    except KeyboardInterrupt:
        lattice_history.status = "Failed"
        await util_save_lattice_history()
        print("Interrupted")
    except Exception as e:
        lattice_history.status = "Failed"
        await util_save_lattice_history()
        raise e


if __name__ == "__main__":
    import asyncio

    asyncio.run(main())



"""Manual chaining (i.e. outside of lattices) of Quarks for RAG ingest"""

import os
from urllib.parse import urlparse

import dotenv
import pandas as pd
from halo import Halo

import quarkupy.lib as q
import quarkupy.lib.context as qc

# Configs
dotenv.load_dotenv()

API_KEY = os.environ.get("QUARK_API_KEY")
API_END_POINT = "https://demo.quarklabs.ai/api/quark"

if API_KEY is None or API_KEY == "":
    raise ValueError("QUARK_API_KEY environment variable not set")

PRINT_DATASETS = False
PRINT_QUARK_METRICS = True
PRINT_QUARK_HISTORY = False
SPINNER = "dots"

host_url = urlparse(API_END_POINT)
url_root = f"{host_url.scheme}://{host_url.netloc}"


async def main():
    print(q.__banner__)

    manager = qc.ContextManager(
        base_url=host_url.geturl()
    )

    with Halo(text="💠 Getting files...", spinner=SPINNER):
        files = await manager.get_files()
        files_df: pd.DataFrame = files.to_pandas()

    print("Files:\n")
    print(files_df)

    with Halo(text="💠 Getting classifiers...", spinner=SPINNER):
        classifiers = await manager.get_classifiers()
        class_df: pd.DataFrame = classifiers.to_pandas()

    print("Classifiers:\n")
    print(class_df)

    with Halo(text="💠 Getting extractors...", spinner=SPINNER):
        extractors = await manager.get_extractors()
        extractors_df = extractors.to_pandas()

    print("Extractors:\n")
    print(extractors_df)

    with Halo(text=f"💠 Getting classifier {class_df.iloc[0].name} text...", spinner=SPINNER):
        classifier_text = await manager.get_classifier_text(class_df.iloc[0].classifier_id)
        classifier_text_df = classifier_text.to_pandas()

    print(f"Classifier {class_df.iloc[0].name} Text:\n")
    print(classifier_text_df)

    with Halo(text=f"💠 Getting classifier {class_df.iloc[0].name} text...", spinner=SPINNER):
        extractor_text = await manager.get_extractor_text(extractors_df.iloc[0].extractor_id)
        extractor_text_df = extractor_text.to_pandas()

    print(f"Extractor {extractors_df.iloc[0].name} Text:\n")
    print(extractor_text_df)


if __name__ == "__main__":
    import asyncio

    asyncio.run(main())



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .admin import (
    AdminResource,
    AsyncAdminResource,
    AdminResourceWithRawResponse,
    AsyncAdminResourceWithRawResponse,
    AdminResourceWithStreamingResponse,
    AsyncAdminResourceWithStreamingResponse,
)
from .users import (
    UsersResource,
    AsyncUsersResource,
    UsersResourceWithRawResponse,
    AsyncUsersResourceWithRawResponse,
    UsersResourceWithStreamingResponse,
    AsyncUsersResourceWithStreamingResponse,
)
from .models import (
    ModelsResource,
    AsyncModelsResource,
    ModelsResourceWithRawResponse,
    AsyncModelsResourceWithRawResponse,
    ModelsResourceWithStreamingResponse,
    AsyncModelsResourceWithStreamingResponse,
)
from .worker import (
    WorkerResource,
    AsyncWorkerResource,
    WorkerResourceWithRawResponse,
    AsyncWorkerResourceWithRawResponse,
    WorkerResourceWithStreamingResponse,
    AsyncWorkerResourceWithStreamingResponse,
)
from .context import (
    ContextResource,
    AsyncContextResource,
    ContextResourceWithRawResponse,
    AsyncContextResourceWithRawResponse,
    ContextResourceWithStreamingResponse,
    AsyncContextResourceWithStreamingResponse,
)
from .dataset import (
    DatasetResource,
    AsyncDatasetResource,
    DatasetResourceWithRawResponse,
    AsyncDatasetResourceWithRawResponse,
    DatasetResourceWithStreamingResponse,
    AsyncDatasetResourceWithStreamingResponse,
)
from .history import (
    HistoryResource,
    AsyncHistoryResource,
    HistoryResourceWithRawResponse,
    AsyncHistoryResourceWithRawResponse,
    HistoryResourceWithStreamingResponse,
    AsyncHistoryResourceWithStreamingResponse,
)
from .profile import (
    ProfileResource,
    AsyncProfileResource,
    ProfileResourceWithRawResponse,
    AsyncProfileResourceWithRawResponse,
    ProfileResourceWithStreamingResponse,
    AsyncProfileResourceWithStreamingResponse,
)
from .sources import (
    SourcesResource,
    AsyncSourcesResource,
    SourcesResourceWithRawResponse,
    AsyncSourcesResourceWithRawResponse,
    SourcesResourceWithStreamingResponse,
    AsyncSourcesResourceWithStreamingResponse,
)
from .authorize import (
    AuthorizeResource,
    AsyncAuthorizeResource,
    AuthorizeResourceWithRawResponse,
    AsyncAuthorizeResourceWithRawResponse,
    AuthorizeResourceWithStreamingResponse,
    AsyncAuthorizeResourceWithStreamingResponse,
)
from .json_schemas import (
    JsonSchemasResource,
    AsyncJsonSchemasResource,
    JsonSchemasResourceWithRawResponse,
    AsyncJsonSchemasResourceWithRawResponse,
    JsonSchemasResourceWithStreamingResponse,
    AsyncJsonSchemasResourceWithStreamingResponse,
)

__all__ = [
    "ContextResource",
    "AsyncContextResource",
    "ContextResourceWithRawResponse",
    "AsyncContextResourceWithRawResponse",
    "ContextResourceWithStreamingResponse",
    "AsyncContextResourceWithStreamingResponse",
    "AdminResource",
    "AsyncAdminResource",
    "AdminResourceWithRawResponse",
    "AsyncAdminResourceWithRawResponse",
    "AdminResourceWithStreamingResponse",
    "AsyncAdminResourceWithStreamingResponse",
    "ModelsResource",
    "AsyncModelsResource",
    "ModelsResourceWithRawResponse",
    "AsyncModelsResourceWithRawResponse",
    "ModelsResourceWithStreamingResponse",
    "AsyncModelsResourceWithStreamingResponse",
    "AuthorizeResource",
    "AsyncAuthorizeResource",
    "AuthorizeResourceWithRawResponse",
    "AsyncAuthorizeResourceWithRawResponse",
    "AuthorizeResourceWithStreamingResponse",
    "AsyncAuthorizeResourceWithStreamingResponse",
    "ProfileResource",
    "AsyncProfileResource",
    "ProfileResourceWithRawResponse",
    "AsyncProfileResourceWithRawResponse",
    "ProfileResourceWithStreamingResponse",
    "AsyncProfileResourceWithStreamingResponse",
    "SourcesResource",
    "AsyncSourcesResource",
    "SourcesResourceWithRawResponse",
    "AsyncSourcesResourceWithRawResponse",
    "SourcesResourceWithStreamingResponse",
    "AsyncSourcesResourceWithStreamingResponse",
    "UsersResource",
    "AsyncUsersResource",
    "UsersResourceWithRawResponse",
    "AsyncUsersResourceWithRawResponse",
    "UsersResourceWithStreamingResponse",
    "AsyncUsersResourceWithStreamingResponse",
    "JsonSchemasResource",
    "AsyncJsonSchemasResource",
    "JsonSchemasResourceWithRawResponse",
    "AsyncJsonSchemasResourceWithRawResponse",
    "JsonSchemasResourceWithStreamingResponse",
    "AsyncJsonSchemasResourceWithStreamingResponse",
    "HistoryResource",
    "AsyncHistoryResource",
    "HistoryResourceWithRawResponse",
    "AsyncHistoryResourceWithRawResponse",
    "HistoryResourceWithStreamingResponse",
    "AsyncHistoryResourceWithStreamingResponse",
    "DatasetResource",
    "AsyncDatasetResource",
    "DatasetResourceWithRawResponse",
    "AsyncDatasetResourceWithRawResponse",
    "DatasetResourceWithStreamingResponse",
    "AsyncDatasetResourceWithStreamingResponse",
    "WorkerResource",
    "AsyncWorkerResource",
    "WorkerResourceWithRawResponse",
    "AsyncWorkerResourceWithRawResponse",
    "WorkerResourceWithStreamingResponse",
    "AsyncWorkerResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .agent import (
    AgentResource,
    AsyncAgentResource,
    AgentResourceWithRawResponse,
    AsyncAgentResourceWithRawResponse,
    AgentResourceWithStreamingResponse,
    AsyncAgentResourceWithStreamingResponse,
)
from .source import (
    SourceResource,
    AsyncSourceResource,
    SourceResourceWithRawResponse,
    AsyncSourceResourceWithRawResponse,
    SourceResourceWithStreamingResponse,
    AsyncSourceResourceWithStreamingResponse,
)
from ..._compat import cached_property
from .management import (
    ManagementResource,
    AsyncManagementResource,
    ManagementResourceWithRawResponse,
    AsyncManagementResourceWithRawResponse,
    ManagementResourceWithStreamingResponse,
    AsyncManagementResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from .context.context import (
    ContextResource,
    AsyncContextResource,
    ContextResourceWithRawResponse,
    AsyncContextResourceWithRawResponse,
    ContextResourceWithStreamingResponse,
    AsyncContextResourceWithStreamingResponse,
)
from .registry.registry import (
    RegistryResource,
    AsyncRegistryResource,
    RegistryResourceWithRawResponse,
    AsyncRegistryResourceWithRawResponse,
    RegistryResourceWithStreamingResponse,
    AsyncRegistryResourceWithStreamingResponse,
)

__all__ = ["WorkerResource", "AsyncWorkerResource"]


class WorkerResource(SyncAPIResource):
    @cached_property
    def management(self) -> ManagementResource:
        return ManagementResource(self._client)

    @cached_property
    def agent(self) -> AgentResource:
        return AgentResource(self._client)

    @cached_property
    def registry(self) -> RegistryResource:
        return RegistryResource(self._client)

    @cached_property
    def source(self) -> SourceResource:
        return SourceResource(self._client)

    @cached_property
    def context(self) -> ContextResource:
        return ContextResource(self._client)

    @cached_property
    def with_raw_response(self) -> WorkerResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return WorkerResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> WorkerResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return WorkerResourceWithStreamingResponse(self)


class AsyncWorkerResource(AsyncAPIResource):
    @cached_property
    def management(self) -> AsyncManagementResource:
        return AsyncManagementResource(self._client)

    @cached_property
    def agent(self) -> AsyncAgentResource:
        return AsyncAgentResource(self._client)

    @cached_property
    def registry(self) -> AsyncRegistryResource:
        return AsyncRegistryResource(self._client)

    @cached_property
    def source(self) -> AsyncSourceResource:
        return AsyncSourceResource(self._client)

    @cached_property
    def context(self) -> AsyncContextResource:
        return AsyncContextResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncWorkerResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return AsyncWorkerResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncWorkerResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return AsyncWorkerResourceWithStreamingResponse(self)


class WorkerResourceWithRawResponse:
    def __init__(self, worker: WorkerResource) -> None:
        self._worker = worker

    @cached_property
    def management(self) -> ManagementResourceWithRawResponse:
        return ManagementResourceWithRawResponse(self._worker.management)

    @cached_property
    def agent(self) -> AgentResourceWithRawResponse:
        return AgentResourceWithRawResponse(self._worker.agent)

    @cached_property
    def registry(self) -> RegistryResourceWithRawResponse:
        return RegistryResourceWithRawResponse(self._worker.registry)

    @cached_property
    def source(self) -> SourceResourceWithRawResponse:
        return SourceResourceWithRawResponse(self._worker.source)

    @cached_property
    def context(self) -> ContextResourceWithRawResponse:
        return ContextResourceWithRawResponse(self._worker.context)


class AsyncWorkerResourceWithRawResponse:
    def __init__(self, worker: AsyncWorkerResource) -> None:
        self._worker = worker

    @cached_property
    def management(self) -> AsyncManagementResourceWithRawResponse:
        return AsyncManagementResourceWithRawResponse(self._worker.management)

    @cached_property
    def agent(self) -> AsyncAgentResourceWithRawResponse:
        return AsyncAgentResourceWithRawResponse(self._worker.agent)

    @cached_property
    def registry(self) -> AsyncRegistryResourceWithRawResponse:
        return AsyncRegistryResourceWithRawResponse(self._worker.registry)

    @cached_property
    def source(self) -> AsyncSourceResourceWithRawResponse:
        return AsyncSourceResourceWithRawResponse(self._worker.source)

    @cached_property
    def context(self) -> AsyncContextResourceWithRawResponse:
        return AsyncContextResourceWithRawResponse(self._worker.context)


class WorkerResourceWithStreamingResponse:
    def __init__(self, worker: WorkerResource) -> None:
        self._worker = worker

    @cached_property
    def management(self) -> ManagementResourceWithStreamingResponse:
        return ManagementResourceWithStreamingResponse(self._worker.management)

    @cached_property
    def agent(self) -> AgentResourceWithStreamingResponse:
        return AgentResourceWithStreamingResponse(self._worker.agent)

    @cached_property
    def registry(self) -> RegistryResourceWithStreamingResponse:
        return RegistryResourceWithStreamingResponse(self._worker.registry)

    @cached_property
    def source(self) -> SourceResourceWithStreamingResponse:
        return SourceResourceWithStreamingResponse(self._worker.source)

    @cached_property
    def context(self) -> ContextResourceWithStreamingResponse:
        return ContextResourceWithStreamingResponse(self._worker.context)


class AsyncWorkerResourceWithStreamingResponse:
    def __init__(self, worker: AsyncWorkerResource) -> None:
        self._worker = worker

    @cached_property
    def management(self) -> AsyncManagementResourceWithStreamingResponse:
        return AsyncManagementResourceWithStreamingResponse(self._worker.management)

    @cached_property
    def agent(self) -> AsyncAgentResourceWithStreamingResponse:
        return AsyncAgentResourceWithStreamingResponse(self._worker.agent)

    @cached_property
    def registry(self) -> AsyncRegistryResourceWithStreamingResponse:
        return AsyncRegistryResourceWithStreamingResponse(self._worker.registry)

    @cached_property
    def source(self) -> AsyncSourceResourceWithStreamingResponse:
        return AsyncSourceResourceWithStreamingResponse(self._worker.source)

    @cached_property
    def context(self) -> AsyncContextResourceWithStreamingResponse:
        return AsyncContextResourceWithStreamingResponse(self._worker.context)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .docling_chunker_run_params import DoclingChunkerRunParams as DoclingChunkerRunParams
from .handlebars_base_run_params import HandlebarsBaseRunParams as HandlebarsBaseRunParams
from .parse_extractor_llm_run_params import ParseExtractorLlmRunParams as ParseExtractorLlmRunParams
from .parse_classifier_llm_run_params import ParseClassifierLlmRunParams as ParseClassifierLlmRunParams
from .onnx_sat_segmentation_run_params import OnnxSatSegmentationRunParams as OnnxSatSegmentationRunParams
from .context_extract_prompt_run_params import ContextExtractPromptRunParams as ContextExtractPromptRunParams
from .context_classifier_prompt_run_params import ContextClassifierPromptRunParams as ContextClassifierPromptRunParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ...._models import BaseModel
from .lattice_react_flow_pos import LatticeReactFlowPos

__all__ = ["LatticeRetrieveFlowResponse", "Edge", "Node", "NodeData", "NodePosition"]


class Edge(BaseModel):
    id: str
    """A unique identifier for the edge."""

    animated: bool
    """A boolean indicating whether this edge has an animation effect."""

    source: str
    """The identifier of the source node this edge connects from."""

    target: str
    """The identifier of the target node this edge connects to."""

    type: str

    style: Optional[object] = None
    """Optional style properties for customizing the edge's appearance."""


class NodeData(BaseModel):
    label: str
    """The label displayed for the node."""


class NodePosition(BaseModel):
    x: int
    """The horizontal position (in pixels)."""

    y: int
    """The vertical position (in pixels)."""


class Node(BaseModel):
    id: str
    """A unique identifier for the node."""

    data: NodeData
    """Represents the data for a React Flow Node in the Lattice graph.

    # Fields

    - `label` - The label displayed for the node.
    """

    position: NodePosition
    """Represents the position of a React Flow Node in the Lattice graph.

    # Fields

    - `x` - The horizontal position (in pixels).
    - `y` - The vertical position (in pixels).
    """

    source_handle_position: LatticeReactFlowPos
    """Source Handle Position"""

    target_handle_position: LatticeReactFlowPos
    """Target Handle Position"""

    type: str
    """The type of the node (serialized as "type")."""

    style: Optional[object] = None
    """Optional style properties for configuring the appearance of the node."""


class LatticeRetrieveFlowResponse(BaseModel):
    description: str
    """A brief description of the graph."""

    edges: List[Edge]
    """A list of edges connecting the nodes in the graph."""

    name: str
    """The name of the graph."""

    nodes: List[Node]
    """A list of nodes in the graph."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .source import Source as Source
from .ml_model import MlModel as MlModel
from .data_set_info import DataSetInfo as DataSetInfo
from .model_list_response import ModelListResponse as ModelListResponse
from .model_update_params import ModelUpdateParams as ModelUpdateParams
from .source_list_response import SourceListResponse as SourceListResponse
from .source_update_params import SourceUpdateParams as SourceUpdateParams
from .dataset_list_response import DatasetListResponse as DatasetListResponse
from .history_list_response import HistoryListResponse as HistoryListResponse
from .authorize_retrieve_params import AuthorizeRetrieveParams as AuthorizeRetrieveParams
from .history_list_flows_params import HistoryListFlowsParams as HistoryListFlowsParams
from .json_schema_list_response import JsonSchemaListResponse as JsonSchemaListResponse
from .history_list_quarks_params import HistoryListQuarksParams as HistoryListQuarksParams
from .dataset_retrieve_csv_params import DatasetRetrieveCsvParams as DatasetRetrieveCsvParams
from .history_list_flows_response import HistoryListFlowsResponse as HistoryListFlowsResponse
from .model_update_partial_params import ModelUpdatePartialParams as ModelUpdatePartialParams
from .dataset_retrieve_json_params import DatasetRetrieveJsonParams as DatasetRetrieveJsonParams
from .history_list_quarks_response import HistoryListQuarksResponse as HistoryListQuarksResponse
from .source_update_partial_params import SourceUpdatePartialParams as SourceUpdatePartialParams
from .dataset_retrieve_arrow_params import DatasetRetrieveArrowParams as DatasetRetrieveArrowParams
from .dataset_retrieve_csv_response import DatasetRetrieveCsvResponse as DatasetRetrieveCsvResponse
from .dataset_retrieve_files_params import DatasetRetrieveFilesParams as DatasetRetrieveFilesParams
from .dataset_retrieve_chunks_params import DatasetRetrieveChunksParams as DatasetRetrieveChunksParams
from .dataset_retrieve_json_response import DatasetRetrieveJsonResponse as DatasetRetrieveJsonResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

__all__ = ["SchemaInfoParam", "Field"]


class Field(TypedDict, total=False):
    data_type: Required[str]

    name: Required[str]

    description: str


class SchemaInfoParam(TypedDict, total=False):
    extra_fields_allowed: Required[bool]

    fields: Required[Iterable[Field]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["APIKeyUpdateResponse"]


class APIKeyUpdateResponse(BaseModel):
    key: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime

import httpx

from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._base_client import make_request_options
from ...types.models import role_update_params, role_update_partial_params
from ...types.models.ml_model_role import MlModelRole
from ...types.models.role_list_response import RoleListResponse
from ...types.context.success_response_message import SuccessResponseMessage

__all__ = ["RolesResource", "AsyncRolesResource"]


class RolesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> RolesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return RolesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RolesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return RolesResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MlModelRole:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._get(
            f"/models/roles/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MlModelRole,
        )

    def update(
        self,
        *,
        config_override: object,
        model_id: str,
        name: str,
        owned_by_identity_id: str,
        description: str | NotGiven = NOT_GIVEN,
        updated_at: Union[str, datetime] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MlModelRole:
        """
        Args:
          config_override: # Optional Override for Model Configuration

          model_id: # Associated Model

          name: # Model Role Name

          owned_by_identity_id: # Model Role Owner

          description: # Description

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._put(
            "/models/roles",
            body=maybe_transform(
                {
                    "config_override": config_override,
                    "model_id": model_id,
                    "name": name,
                    "owned_by_identity_id": owned_by_identity_id,
                    "description": description,
                    "updated_at": updated_at,
                },
                role_update_params.RoleUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MlModelRole,
        )

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RoleListResponse:
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._get(
            "/models/roles",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RoleListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SuccessResponseMessage:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._delete(
            f"/models/roles/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SuccessResponseMessage,
        )

    def retrieve_schema(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._get(
            "/models/roles/schema",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def update_partial(
        self,
        id: str,
        *,
        config_override: object,
        model_id: str,
        name: str,
        owned_by_identity_id: str,
        description: str | NotGiven = NOT_GIVEN,
        updated_at: Union[str, datetime] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MlModelRole:
        """
        Args:
          config_override: # Optional Override for Model Configuration

          model_id: # Associated Model

          name: # Model Role Name

          owned_by_identity_id: # Model Role Owner

          description: # Description

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._patch(
            f"/models/roles/{id}",
            body=maybe_transform(
                {
                    "config_override": config_override,
                    "model_id": model_id,
                    "name": name,
                    "owned_by_identity_id": owned_by_identity_id,
                    "description": description,
                    "updated_at": updated_at,
                },
                role_update_partial_params.RoleUpdatePartialParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MlModelRole,
        )


class AsyncRolesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncRolesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return AsyncRolesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRolesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return AsyncRolesResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MlModelRole:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._get(
            f"/models/roles/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MlModelRole,
        )

    async def update(
        self,
        *,
        config_override: object,
        model_id: str,
        name: str,
        owned_by_identity_id: str,
        description: str | NotGiven = NOT_GIVEN,
        updated_at: Union[str, datetime] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MlModelRole:
        """
        Args:
          config_override: # Optional Override for Model Configuration

          model_id: # Associated Model

          name: # Model Role Name

          owned_by_identity_id: # Model Role Owner

          description: # Description

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._put(
            "/models/roles",
            body=await async_maybe_transform(
                {
                    "config_override": config_override,
                    "model_id": model_id,
                    "name": name,
                    "owned_by_identity_id": owned_by_identity_id,
                    "description": description,
                    "updated_at": updated_at,
                },
                role_update_params.RoleUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MlModelRole,
        )

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RoleListResponse:
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._get(
            "/models/roles",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RoleListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SuccessResponseMessage:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._delete(
            f"/models/roles/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SuccessResponseMessage,
        )

    async def retrieve_schema(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> object:
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._get(
            "/models/roles/schema",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def update_partial(
        self,
        id: str,
        *,
        config_override: object,
        model_id: str,
        name: str,
        owned_by_identity_id: str,
        description: str | NotGiven = NOT_GIVEN,
        updated_at: Union[str, datetime] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> MlModelRole:
        """
        Args:
          config_override: # Optional Override for Model Configuration

          model_id: # Associated Model

          name: # Model Role Name

          owned_by_identity_id: # Model Role Owner

          description: # Description

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._patch(
            f"/models/roles/{id}",
            body=await async_maybe_transform(
                {
                    "config_override": config_override,
                    "model_id": model_id,
                    "name": name,
                    "owned_by_identity_id": owned_by_identity_id,
                    "description": description,
                    "updated_at": updated_at,
                },
                role_update_partial_params.RoleUpdatePartialParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=MlModelRole,
        )


class RolesResourceWithRawResponse:
    def __init__(self, roles: RolesResource) -> None:
        self._roles = roles

        self.retrieve = to_raw_response_wrapper(
            roles.retrieve,
        )
        self.update = to_raw_response_wrapper(
            roles.update,
        )
        self.list = to_raw_response_wrapper(
            roles.list,
        )
        self.delete = to_raw_response_wrapper(
            roles.delete,
        )
        self.retrieve_schema = to_raw_response_wrapper(
            roles.retrieve_schema,
        )
        self.update_partial = to_raw_response_wrapper(
            roles.update_partial,
        )


class AsyncRolesResourceWithRawResponse:
    def __init__(self, roles: AsyncRolesResource) -> None:
        self._roles = roles

        self.retrieve = async_to_raw_response_wrapper(
            roles.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            roles.update,
        )
        self.list = async_to_raw_response_wrapper(
            roles.list,
        )
        self.delete = async_to_raw_response_wrapper(
            roles.delete,
        )
        self.retrieve_schema = async_to_raw_response_wrapper(
            roles.retrieve_schema,
        )
        self.update_partial = async_to_raw_response_wrapper(
            roles.update_partial,
        )


class RolesResourceWithStreamingResponse:
    def __init__(self, roles: RolesResource) -> None:
        self._roles = roles

        self.retrieve = to_streamed_response_wrapper(
            roles.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            roles.update,
        )
        self.list = to_streamed_response_wrapper(
            roles.list,
        )
        self.delete = to_streamed_response_wrapper(
            roles.delete,
        )
        self.retrieve_schema = to_streamed_response_wrapper(
            roles.retrieve_schema,
        )
        self.update_partial = to_streamed_response_wrapper(
            roles.update_partial,
        )


class AsyncRolesResourceWithStreamingResponse:
    def __init__(self, roles: AsyncRolesResource) -> None:
        self._roles = roles

        self.retrieve = async_to_streamed_response_wrapper(
            roles.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            roles.update,
        )
        self.list = async_to_streamed_response_wrapper(
            roles.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            roles.delete,
        )
        self.retrieve_schema = async_to_streamed_response_wrapper(
            roles.retrieve_schema,
        )
        self.update_partial = async_to_streamed_response_wrapper(
            roles.update_partial,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .agent import (
    AgentResource,
    AsyncAgentResource,
    AgentResourceWithRawResponse,
    AsyncAgentResourceWithRawResponse,
    AgentResourceWithStreamingResponse,
    AsyncAgentResourceWithStreamingResponse,
)
from .source import (
    SourceResource,
    AsyncSourceResource,
    SourceResourceWithRawResponse,
    AsyncSourceResourceWithRawResponse,
    SourceResourceWithStreamingResponse,
    AsyncSourceResourceWithStreamingResponse,
)
from .worker import (
    WorkerResource,
    AsyncWorkerResource,
    WorkerResourceWithRawResponse,
    AsyncWorkerResourceWithRawResponse,
    WorkerResourceWithStreamingResponse,
    AsyncWorkerResourceWithStreamingResponse,
)
from .context import (
    ContextResource,
    AsyncContextResource,
    ContextResourceWithRawResponse,
    AsyncContextResourceWithRawResponse,
    ContextResourceWithStreamingResponse,
    AsyncContextResourceWithStreamingResponse,
)
from .registry import (
    RegistryResource,
    AsyncRegistryResource,
    RegistryResourceWithRawResponse,
    AsyncRegistryResourceWithRawResponse,
    RegistryResourceWithStreamingResponse,
    AsyncRegistryResourceWithStreamingResponse,
)
from .management import (
    ManagementResource,
    AsyncManagementResource,
    ManagementResourceWithRawResponse,
    AsyncManagementResourceWithRawResponse,
    ManagementResourceWithStreamingResponse,
    AsyncManagementResourceWithStreamingResponse,
)

__all__ = [
    "ManagementResource",
    "AsyncManagementResource",
    "ManagementResourceWithRawResponse",
    "AsyncManagementResourceWithRawResponse",
    "ManagementResourceWithStreamingResponse",
    "AsyncManagementResourceWithStreamingResponse",
    "AgentResource",
    "AsyncAgentResource",
    "AgentResourceWithRawResponse",
    "AsyncAgentResourceWithRawResponse",
    "AgentResourceWithStreamingResponse",
    "AsyncAgentResourceWithStreamingResponse",
    "RegistryResource",
    "AsyncRegistryResource",
    "RegistryResourceWithRawResponse",
    "AsyncRegistryResourceWithRawResponse",
    "RegistryResourceWithStreamingResponse",
    "AsyncRegistryResourceWithStreamingResponse",
    "SourceResource",
    "AsyncSourceResource",
    "SourceResourceWithRawResponse",
    "AsyncSourceResourceWithRawResponse",
    "SourceResourceWithStreamingResponse",
    "AsyncSourceResourceWithStreamingResponse",
    "ContextResource",
    "AsyncContextResource",
    "ContextResourceWithRawResponse",
    "AsyncContextResourceWithRawResponse",
    "ContextResourceWithStreamingResponse",
    "AsyncContextResourceWithStreamingResponse",
    "WorkerResource",
    "AsyncWorkerResource",
    "WorkerResourceWithRawResponse",
    "AsyncWorkerResourceWithRawResponse",
    "WorkerResourceWithStreamingResponse",
    "AsyncWorkerResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["SourceCreateParams", "Config"]


class SourceCreateParams(TypedDict, total=False):
    config: Required[Config]

    name: Required[str]

    description: str


class Config(TypedDict, total=False):
    access_key_id: Required[str]

    endpoint: Required[str]

    region: Required[str]

    secret_access_key: Required[str]

    url: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel
from .schema_info import SchemaInfo

__all__ = ["DescribedInputField"]


class DescribedInputField(BaseModel):
    advanced: bool

    field_name: str

    field_type: str

    friendly_name: str

    required: bool

    sensitive: bool

    default_value: Optional[object] = None

    description: Optional[str] = None

    ipc_schema: Optional[SchemaInfo] = None
    """API-Friendly representation of a [Schema]"""



from __future__ import annotations, absolute_import

from quarkupy.types.history import FlowHistoryItem, QuarkHistoryItem
from quarkupy.types.worker.registry import QuarkRegistryItem


from .driver import *
from .inputs import *
from .quarks import *

LatticeStatus = Literal["New", "Scheduled", "Running", "Completed", "Failed"]
QuarkStatus = Literal["New", "Scheduled", "Running", "OutputStaged", "Completed", "Failed"]

__all__ = [
    "QuarkRemoteDriver",
    "LatticeStatus",
    "QuarkStatus",
    "QuarkInput",
    "QuarkHistoryItem",
    "QuarkRegistryItem",
    "FlowHistoryItem",
    "ContextClassifierPromptInput",
    "ContextExtractPromptInput",
    "ParseClassifierLlmInput",
    "ParseExtractorLlmInput",
    "S3ReadCSVQuarkInput",
    "S3ReadWholeFileQuarkInput",
    "DocExtractQuarkInput",
    "TextTemplateInput",
    "OpenAIEmbeddingsInput",
    "OpenAICompletionsInput",
    "DocChunkerQuarkInput",
    "SnowflakeReadInput",
    "VectorDBIngestInput",
    "VectorDBSearchInput",
    "OpenAICompletionBaseQuark",
    "OpenAIEmbeddingsQuark",
    "OpendalInput",
    "ConfigOpendalConfigInputOpendalS3Config",
    "ConfigOpendalConfigInputOpendalMemoryConfig",
    "ConfigOpendalConfigInputOpendalGDriveConfig",
    "ConfigOpendalConfigInputOpendalOneDriveConfig",
    "OpendalReadQuark",
    "DocExtractQuark",
    "DocChunkerQuark",
    "VectorDBIngestQuark",
    "VectorDBSearchQuark",
    "S3ReadCSVQuark",
    "S3ReadWholeFileQuark",
    "TextTemplateBaseQuark",
    "DocChunkerQuark",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel

__all__ = ["ManagementRetrievePythonStatusResponse"]


class ManagementRetrievePythonStatusResponse(BaseModel):
    exec_prefix: str
    """Python exec prefix"""

    modules: List[str]
    """Available modules"""

    path: List[str]
    """Python path"""

    prefix: str
    """Python path prefix"""

    site: List[str]
    """Python site"""

    version: str
    """Python version"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from .lattice import (
    LatticeResource,
    AsyncLatticeResource,
    LatticeResourceWithRawResponse,
    AsyncLatticeResourceWithRawResponse,
    LatticeResourceWithStreamingResponse,
    AsyncLatticeResourceWithStreamingResponse,
)
from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._compat import cached_property
from .quark.quark import (
    QuarkResource,
    AsyncQuarkResource,
    QuarkResourceWithRawResponse,
    AsyncQuarkResourceWithRawResponse,
    QuarkResourceWithStreamingResponse,
    AsyncQuarkResourceWithStreamingResponse,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.worker.registry_list_response import RegistryListResponse

__all__ = ["RegistryResource", "AsyncRegistryResource"]


class RegistryResource(SyncAPIResource):
    @cached_property
    def quark(self) -> QuarkResource:
        return QuarkResource(self._client)

    @cached_property
    def lattice(self) -> LatticeResource:
        return LatticeResource(self._client)

    @cached_property
    def with_raw_response(self) -> RegistryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return RegistryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RegistryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return RegistryResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RegistryListResponse:
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._get(
            "/worker/registry",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RegistryListResponse,
        )


class AsyncRegistryResource(AsyncAPIResource):
    @cached_property
    def quark(self) -> AsyncQuarkResource:
        return AsyncQuarkResource(self._client)

    @cached_property
    def lattice(self) -> AsyncLatticeResource:
        return AsyncLatticeResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncRegistryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return AsyncRegistryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRegistryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return AsyncRegistryResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> RegistryListResponse:
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._get(
            "/worker/registry",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=RegistryListResponse,
        )


class RegistryResourceWithRawResponse:
    def __init__(self, registry: RegistryResource) -> None:
        self._registry = registry

        self.list = to_raw_response_wrapper(
            registry.list,
        )

    @cached_property
    def quark(self) -> QuarkResourceWithRawResponse:
        return QuarkResourceWithRawResponse(self._registry.quark)

    @cached_property
    def lattice(self) -> LatticeResourceWithRawResponse:
        return LatticeResourceWithRawResponse(self._registry.lattice)


class AsyncRegistryResourceWithRawResponse:
    def __init__(self, registry: AsyncRegistryResource) -> None:
        self._registry = registry

        self.list = async_to_raw_response_wrapper(
            registry.list,
        )

    @cached_property
    def quark(self) -> AsyncQuarkResourceWithRawResponse:
        return AsyncQuarkResourceWithRawResponse(self._registry.quark)

    @cached_property
    def lattice(self) -> AsyncLatticeResourceWithRawResponse:
        return AsyncLatticeResourceWithRawResponse(self._registry.lattice)


class RegistryResourceWithStreamingResponse:
    def __init__(self, registry: RegistryResource) -> None:
        self._registry = registry

        self.list = to_streamed_response_wrapper(
            registry.list,
        )

    @cached_property
    def quark(self) -> QuarkResourceWithStreamingResponse:
        return QuarkResourceWithStreamingResponse(self._registry.quark)

    @cached_property
    def lattice(self) -> LatticeResourceWithStreamingResponse:
        return LatticeResourceWithStreamingResponse(self._registry.lattice)


class AsyncRegistryResourceWithStreamingResponse:
    def __init__(self, registry: AsyncRegistryResource) -> None:
        self._registry = registry

        self.list = async_to_streamed_response_wrapper(
            registry.list,
        )

    @cached_property
    def quark(self) -> AsyncQuarkResourceWithStreamingResponse:
        return AsyncQuarkResourceWithStreamingResponse(self._registry.quark)

    @cached_property
    def lattice(self) -> AsyncLatticeResourceWithStreamingResponse:
        return AsyncLatticeResourceWithStreamingResponse(self._registry.lattice)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Iterable
from typing_extensions import Literal

import httpx

from ...._types import NOT_GIVEN, Body, Query, Headers, NotGiven
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from ....types.worker.registry import lattice_update_register_params
from ....types.worker.registry.quark_tag import QuarkTag
from ....types.context.success_response_message import SuccessResponseMessage
from ....types.worker.registry.schema_info_param import SchemaInfoParam
from ....types.worker.registry.lattice_registry_item import LatticeRegistryItem
from ....types.worker.registry.described_input_field_param import DescribedInputFieldParam
from ....types.worker.registry.lattice_retrieve_flow_response import LatticeRetrieveFlowResponse

__all__ = ["LatticeResource", "AsyncLatticeResource"]


class LatticeResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LatticeResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return LatticeResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LatticeResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return LatticeResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LatticeRegistryItem:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._get(
            f"/worker/registry/lattice/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LatticeRegistryItem,
        )

    def retrieve_flow(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LatticeRetrieveFlowResponse:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._get(
            f"/worker/registry/lattice/{id}/flow",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LatticeRetrieveFlowResponse,
        )

    def update_register(
        self,
        *,
        author: str,
        edges: Iterable[lattice_update_register_params.Edge],
        flow_registry_id: str,
        hidden: bool,
        identifier: str,
        inputs: Iterable[DescribedInputFieldParam],
        lattice_type: Literal["Ingest", "Inference", "Other"],
        name: str,
        nodes: Iterable[lattice_update_register_params.Node],
        tags: List[QuarkTag],
        version: str,
        description: str | NotGiven = NOT_GIVEN,
        output_schema: SchemaInfoParam | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SuccessResponseMessage:
        """
        Args:
          output_schema: API-Friendly representation of a [Schema]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return self._put(
            "/worker/registry/lattice/register",
            body=maybe_transform(
                {
                    "author": author,
                    "edges": edges,
                    "flow_registry_id": flow_registry_id,
                    "hidden": hidden,
                    "identifier": identifier,
                    "inputs": inputs,
                    "lattice_type": lattice_type,
                    "name": name,
                    "nodes": nodes,
                    "tags": tags,
                    "version": version,
                    "description": description,
                    "output_schema": output_schema,
                },
                lattice_update_register_params.LatticeUpdateRegisterParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SuccessResponseMessage,
        )


class AsyncLatticeResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLatticeResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return AsyncLatticeResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLatticeResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return AsyncLatticeResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LatticeRegistryItem:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._get(
            f"/worker/registry/lattice/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LatticeRegistryItem,
        )

    async def retrieve_flow(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> LatticeRetrieveFlowResponse:
        """
        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._get(
            f"/worker/registry/lattice/{id}/flow",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=LatticeRetrieveFlowResponse,
        )

    async def update_register(
        self,
        *,
        author: str,
        edges: Iterable[lattice_update_register_params.Edge],
        flow_registry_id: str,
        hidden: bool,
        identifier: str,
        inputs: Iterable[DescribedInputFieldParam],
        lattice_type: Literal["Ingest", "Inference", "Other"],
        name: str,
        nodes: Iterable[lattice_update_register_params.Node],
        tags: List[QuarkTag],
        version: str,
        description: str | NotGiven = NOT_GIVEN,
        output_schema: SchemaInfoParam | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> SuccessResponseMessage:
        """
        Args:
          output_schema: API-Friendly representation of a [Schema]

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        extra_headers = {"Accept": "application/json; charset=utf-8", **(extra_headers or {})}
        return await self._put(
            "/worker/registry/lattice/register",
            body=await async_maybe_transform(
                {
                    "author": author,
                    "edges": edges,
                    "flow_registry_id": flow_registry_id,
                    "hidden": hidden,
                    "identifier": identifier,
                    "inputs": inputs,
                    "lattice_type": lattice_type,
                    "name": name,
                    "nodes": nodes,
                    "tags": tags,
                    "version": version,
                    "description": description,
                    "output_schema": output_schema,
                },
                lattice_update_register_params.LatticeUpdateRegisterParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SuccessResponseMessage,
        )


class LatticeResourceWithRawResponse:
    def __init__(self, lattice: LatticeResource) -> None:
        self._lattice = lattice

        self.retrieve = to_raw_response_wrapper(
            lattice.retrieve,
        )
        self.retrieve_flow = to_raw_response_wrapper(
            lattice.retrieve_flow,
        )
        self.update_register = to_raw_response_wrapper(
            lattice.update_register,
        )


class AsyncLatticeResourceWithRawResponse:
    def __init__(self, lattice: AsyncLatticeResource) -> None:
        self._lattice = lattice

        self.retrieve = async_to_raw_response_wrapper(
            lattice.retrieve,
        )
        self.retrieve_flow = async_to_raw_response_wrapper(
            lattice.retrieve_flow,
        )
        self.update_register = async_to_raw_response_wrapper(
            lattice.update_register,
        )


class LatticeResourceWithStreamingResponse:
    def __init__(self, lattice: LatticeResource) -> None:
        self._lattice = lattice

        self.retrieve = to_streamed_response_wrapper(
            lattice.retrieve,
        )
        self.retrieve_flow = to_streamed_response_wrapper(
            lattice.retrieve_flow,
        )
        self.update_register = to_streamed_response_wrapper(
            lattice.update_register,
        )


class AsyncLatticeResourceWithStreamingResponse:
    def __init__(self, lattice: AsyncLatticeResource) -> None:
        self._lattice = lattice

        self.retrieve = async_to_streamed_response_wrapper(
            lattice.retrieve,
        )
        self.retrieve_flow = async_to_streamed_response_wrapper(
            lattice.retrieve_flow,
        )
        self.update_register = async_to_streamed_response_wrapper(
            lattice.update_register,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["SourceUpdateParams"]


class SourceUpdateParams(TypedDict, total=False):
    config: Required[object]

    config_type: Required[Literal["S3ObjectStore", "Other"]]

    name: Required[str]

    owned_by_identity_id: Required[str]

    source_type: Required[Literal["Files", "Database", "Other"]]

    status: Required[Literal["SetupInProgress", "SetupComplete", "Other"]]

    description: str

    updated_at: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from ......_compat import cached_property
from ......_resource import SyncAPIResource, AsyncAPIResource
from .lancedb_ingest import (
    LancedbIngestResource,
    AsyncLancedbIngestResource,
    LancedbIngestResourceWithRawResponse,
    AsyncLancedbIngestResourceWithRawResponse,
    LancedbIngestResourceWithStreamingResponse,
    AsyncLancedbIngestResourceWithStreamingResponse,
)
from .lancedb_search import (
    LancedbSearchResource,
    AsyncLancedbSearchResource,
    LancedbSearchResourceWithRawResponse,
    AsyncLancedbSearchResourceWithRawResponse,
    LancedbSearchResourceWithStreamingResponse,
    AsyncLancedbSearchResourceWithStreamingResponse,
)

__all__ = ["VectorResource", "AsyncVectorResource"]


class VectorResource(SyncAPIResource):
    @cached_property
    def lancedb_ingest(self) -> LancedbIngestResource:
        return LancedbIngestResource(self._client)

    @cached_property
    def lancedb_search(self) -> LancedbSearchResource:
        return LancedbSearchResource(self._client)

    @cached_property
    def with_raw_response(self) -> VectorResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return VectorResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> VectorResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return VectorResourceWithStreamingResponse(self)


class AsyncVectorResource(AsyncAPIResource):
    @cached_property
    def lancedb_ingest(self) -> AsyncLancedbIngestResource:
        return AsyncLancedbIngestResource(self._client)

    @cached_property
    def lancedb_search(self) -> AsyncLancedbSearchResource:
        return AsyncLancedbSearchResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncVectorResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/quarklabsai/quarkupy#accessing-raw-response-data-eg-headers
        """
        return AsyncVectorResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncVectorResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/quarklabsai/quarkupy#with_streaming_response
        """
        return AsyncVectorResourceWithStreamingResponse(self)


class VectorResourceWithRawResponse:
    def __init__(self, vector: VectorResource) -> None:
        self._vector = vector

    @cached_property
    def lancedb_ingest(self) -> LancedbIngestResourceWithRawResponse:
        return LancedbIngestResourceWithRawResponse(self._vector.lancedb_ingest)

    @cached_property
    def lancedb_search(self) -> LancedbSearchResourceWithRawResponse:
        return LancedbSearchResourceWithRawResponse(self._vector.lancedb_search)


class AsyncVectorResourceWithRawResponse:
    def __init__(self, vector: AsyncVectorResource) -> None:
        self._vector = vector

    @cached_property
    def lancedb_ingest(self) -> AsyncLancedbIngestResourceWithRawResponse:
        return AsyncLancedbIngestResourceWithRawResponse(self._vector.lancedb_ingest)

    @cached_property
    def lancedb_search(self) -> AsyncLancedbSearchResourceWithRawResponse:
        return AsyncLancedbSearchResourceWithRawResponse(self._vector.lancedb_search)


class VectorResourceWithStreamingResponse:
    def __init__(self, vector: VectorResource) -> None:
        self._vector = vector

    @cached_property
    def lancedb_ingest(self) -> LancedbIngestResourceWithStreamingResponse:
        return LancedbIngestResourceWithStreamingResponse(self._vector.lancedb_ingest)

    @cached_property
    def lancedb_search(self) -> LancedbSearchResourceWithStreamingResponse:
        return LancedbSearchResourceWithStreamingResponse(self._vector.lancedb_search)


class AsyncVectorResourceWithStreamingResponse:
    def __init__(self, vector: AsyncVectorResource) -> None:
        self._vector = vector

    @cached_property
    def lancedb_ingest(self) -> AsyncLancedbIngestResourceWithStreamingResponse:
        return AsyncLancedbIngestResourceWithStreamingResponse(self._vector.lancedb_ingest)

    @cached_property
    def lancedb_search(self) -> AsyncLancedbSearchResourceWithStreamingResponse:
        return AsyncLancedbSearchResourceWithStreamingResponse(self._vector.lancedb_search)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["AgentRetrieveResponse", "Agent", "AgentFlowAgent"]


class AgentFlowAgent(BaseModel):
    id: str


class Agent(BaseModel):
    id: str

    flow_agent: AgentFlowAgent
    """REST API representation of the [FlowAgent]"""

    name: str

    status: Literal["New", "Running", "Stopped", "Paused", "Error"]

    description: Optional[str] = None


class AgentRetrieveResponse(BaseModel):
    agents: List[Agent]
